%% Determination of a minimal set of Elementary Flux Modes (EFM's) %%
% Available online on http://www.inma.ucl.ac.be/~jungers/

%% To run this code you need to instal  Yalmip  and a solver like SEDUMI and
%% make sure that your Matlab version contains the optimisation toolbox
%% (optim)

%% Inputs
% You need to specify matrices
% (i) N: (mxn) full-rank stoichiometric matrix deduced from the metabolic
% network (m: number of the internal balanced metabolites and n: number of
% fluxes).
% (ii) Nm: a (pxn) full-rank matrix with p the number of available
% measurements.
% (iii) vm: a vector containing the specific rates of the p measured species.

%% Methodology Explanation
% vm is projected in a small polytope K of dimension p (n of experimental
% data), and it is decomposed in p extreme rays (yi's):
% vm = a1*y1+a2*y2+...+ap*yp
% K is the projection by Nm of the  polytope S ( described by Nv=0, v>=0).
% If yi is an extreme ray of the small polytope, then yi=Nmei, where ei is
%  an extreme ray of S.
% vm = Nm*v ==> yi = Nm*ei
% As vm has been decomposed in vm=sum(ai*yi), a decomposition in the
% extreme rays of the big cone is also achieved.
% vm = sum(ai*Nm*ei) = Nm*sum(ai*ei)
% Hence, v = sum(ai*ei) is a solution

% The algorithm returns the minimal set of EFMs: a matrix E of p column
% vectors (ei)

%% THIS IS A BETA VERSION
% Please, if you have any remark or comment, I'll be happy to take it.
% Please send an email to raphael.jungers@uclouvain.be


function [E,combili,v]=minimal_efm_decomposition(N,Nm,vm)
epsil=1e-9;
s=size(N);
p=size(vm);
p=p(1)
maxiter=20;%maximum number of iterations to find a vertex


e=sdpvar(s(2),1);
y=sdpvar(p,1);
sumvm=ones(p)*vm;
obj=sdpvar(1,1);
lambda=sdpvar(1,1);
indices=[];
normvm=vm'*vm;%norm of vm to normalize the problem and work in a bounded polytope rather than a cone
vv=vm;%vv corresponds to vi in the paper which is projected on smaller and smaller faces

found=0;
i=0;
while (i<(p))&(found==0)

    i=i+1
    sdpfail=2;
    iter=0;

    while (iter<maxiter)&(sdpfail~=0)%we look for a vertex e of S
        iter=iter+1;

        F=set(N*e==zeros(s(1),1));
        F=F+set(ones(1,s(2))*e==1);
        direc=rand(1,s(2));
        F=F+set(direc*e==obj);
        for j=1:s(2)  %%%we restrict the search to the face of S in which we are working,
            %the face is characterized by ej=0 for all j in "indices"
            if issin(j,indices)
                F=F+set(e(j)==0);
            else
                F=F+set(e(j)>=0);
            end
        end
        diagnostic=solvesdp(F,-obj);
        sdpfail= diagnostic.problem;
    end



    E(:,i)=double(e); %we store the ith vector found
    coe=normvm/(vm'*Nm*E(:,i));
    E(:,i)=coe*E(:,i);
    E(indices,i)=0;
    Y(:,i)=Nm*E(:,i);%yi is ei projected





    % Eentier(:,i)=round(1e13*(E(:,i)/minnonnul(E(:,i))))/1e13;
    % coeff(i)=1/sum(Eentier(:,i));%coeff est le coefficient qui fait passer E de sa vraie valeur  des valeurs entires








    if (norm(Y(:,i)-vv)<epsil)
        found=1;
        disp('vm is finally projected on a vertex: the algorithm is now terminating')
        pause
    else
        if i==p
            disp('there is a problem, the last but one projected vector is not a vertex, the distance to the vertex is')
            norm(Y(:,i)-vv)
            disp('and we ask')
            epsil
            pause
        end

        %%%determination of the new face by projecting vv
        F=set(N*e==zeros(s(1),1));
        F=F+set(Nm*e==y);
        F=F+set(y==vv+lambda*(vv-Y(:,i)));
        for j=1:s(2)  %%%we constraint e to be in the face Pi
            if issin(j,indices)
                F=F+set(e(j)==0);
            else
                F=F+set(e(j)>=0);
            end
        end
        diagnostic=solvesdp(F,-lambda);
        sdpfail= diagnostic.problem;
        vve=double(e);

        indices=[];%indices is the indices correponding to the ej that are identically zero in the face in which we are working
        for j=1:length(vve)
            if (vve(j)<epsil)
                indices=[indices j];
            end
        end
        nbindicesnuls(i)=length(indices);
        vve(indices)=0;
        vv=Nm*vve;
        lambdas(i)=double(lambda);
    end

end



if(found==0)
    disp('something has gone wrong: a decomposition has not been found in p steps')
    i
    pause
    E(:,i+1)=vve;
else
    i=i-1;%i+1 is the number of necessary extreme rays
end
%%%%determination of the coefficients of the linear combination by
%%%%transforming the lambdas
prod=1;
for k=1:(i)

    prod=prod/(1+lambdas(k));
    combili(k)=prod*(lambdas(k));
end
combili(i+1)=prod;
v=E*combili';

pause
check_rays(N,Nm,vm,E')
disp('if the number above is one then the vectors in E are well extreme rays')

norm(Nm*E*combili'-vm)
disp('if the number above is near from zero then the linear combination given by combili is a valid solution')
pause

end


%%

function c=check_rays(n,nm,vm,vvv)
%%%receives a (n x m) matrix n

%%%%%checks that the rows of vvv are well extreme rays of the cone given
%%%%%by n e=0.  returns 1 if it is the case

ep=1E-8;
t=size(n);
s=size(vvv);
c=1;

%%checks the sign
   if min(min(vvv))<-ep
        display('error there is a nonpositive vector')
        pause
    end


for i=1:s(1)%checks the equalities
    if norm([n]*vvv(i,:)')>ep
        [n]*vvv(i,:)';
        c=0;
        norm([n]*vvv(i,:)')
        display('error the following vector violates n v=0')
        i
        vvv(i,:)
        pause
    end
    indicnul=nnull(vvv(i,:),ep);
    r=length(indicnul);
    id=zeros(r,t(2));
    for l=1:r
        id(l,indicnul(l))=1;
    end
    if rank([n;nm;id])~=t(2)
        c=0
        i
        display('error it is not a vertex of the cone: not enough zero entries')
        pause
    end

end
end

%%
%function nnull
function cpt=nnull(v,eps) %counts the number of entries smaller than eps .

cpt=[];
s=length(v);
for j=1:s
    if (abs(v(j))<eps)
        cpt=[cpt;j];
    end
end
end

%%
%function issin

function f=issin(k,vec)
%renvoie1 si k est dans vec, zero sinon
ep=1E-7;
q=size(vec);
f=0;
for i=1:q(2)
   if norm(vec(:,i)-k)<ep
       f=1;
   end
end
end